<?php

function build_mysign($sort_array,$key,$sign_type = "MD5") {
    $prestr = create_linkstring($sort_array);     	//把数组所有元素，按照“参数=参数值”的模式用“&”字符拼接成字符串
    $prestr = $prestr.$key;							//把拼接后的字符串再与安全校验码直接连接起来
    $mysgin = sign($prestr,$sign_type);			    //把最终的字符串签名，获得签名结果
    return $mysgin;
}	

/********************************************************************************/

/**把数组所有元素，按照“参数=参数值”的模式用“&”字符拼接成字符串
 * $array 需要拼接的数组
 * return 拼接完成以后的字符串
 */
function create_linkstring($array) {
    $arg  = "";
    while (list ($key, $val) = each ($array)) {
        $arg.=$key."=".$val."&";
    }
    $arg = substr($arg,0,count($arg)-2);		     //去掉最后一个&字符
    return $arg;
}

/********************************************************************************/

/**除去数组中的空值和签名参数
 * $parameter 签名参数组
 * return 去掉空值与签名参数后的新签名参数组
 */
function para_filter($parameter) {
    $para = array();
    while (list ($key, $val) = each ($parameter)) {
        if($key == "sign" || $key == "sign_type" || $val == "")continue;
        else	$para[$key] = $parameter[$key];
    }
    return $para;
}

/********************************************************************************/

/**对数组排序
 * $array 排序前的数组
 * return 排序后的数组
 */
function arg_sort($array) {
    ksort($array);
    reset($array);
    return $array;
}

/********************************************************************************/

/**签名字符串
 * $prestr 需要签名的字符串
 * $sign_type 签名类型，也就是sec_id
 * return 签名结果
 */
function sign($prestr,$sign_type) {
    $sign='';
    if($sign_type == 'MD5') {
        $sign = md5($prestr);
    }elseif($sign_type =='DSA') {
        //DSA 签名方法待后续开发
        die("DSA 签名方法待后续开发，请先使用MD5签名方式");
    }else {
        die("支付宝暂不支持".$sign_type."类型的签名方式");
    }
    return $sign;
}

/********************************************************************************/

/**日志消息,把支付宝返回的参数记录下来
 * 请注意服务器是否开通fopen配置
 */
function  log_result($word) {
    $fp = fopen("log.txt","a");
    flock($fp, LOCK_EX) ;
    fwrite($fp,"执行日期：".strftime("%Y%m%d%H%M%S",time())."\n".$word."\n");
    flock($fp, LOCK_UN);
    fclose($fp);
}	

/********************************************************************************/

/**实现多种字符编码方式
 *$input 需要编码的字符串
 *$_output_charset 输出的编码格式
 *$_input_charset 输入的编码格式
 *return 编码后的字符串
 */
function charset_encode($input,$_output_charset ,$_input_charset) {
    $output = "";
    if(!isset($_output_charset) )$_output_charset  = $_input_charset;
    if($_input_charset == $_output_charset || $input ==null ) {
        $output = $input;
    } elseif (function_exists("mb_convert_encoding")) {
        $output = mb_convert_encoding($input,$_output_charset,$_input_charset);
    } elseif(function_exists("iconv")) {
        $output = iconv($_input_charset,$_output_charset,$input);
    } else die("sorry, you have no libs support for charset change.");
    return $output;
}

/********************************************************************************/

/** 实现多种字符解码方式
 * $input 需要解码的字符串
 * $_output_charset 输出的解码格式
 * $_input_charset 输入的解码格式
 * return 解码后的字符串
 */
function charset_decode($input,$_input_charset ,$_output_charset) {
    $output = "";
    if(!isset($_input_charset) )$_input_charset  = $_input_charset ;
    if($_input_charset == $_output_charset || $input ==null ) {
        $output = $input;
    } elseif (function_exists("mb_convert_encoding")) {
        $output = mb_convert_encoding($input,$_output_charset,$_input_charset);
    } elseif(function_exists("iconv")) {
        $output = iconv($_input_charset,$_output_charset,$input);
    } else die("sorry, you have no libs support for charset changes.");
    return $output;
}

/********************************************************************************/

/**
 * 通过节点路径返回字符串的某个节点值
 * $res_data——XML 格式字符串
 * 返回节点参数
 */
function getDataForXML($res_data,$node)
{
	$xml = simplexml_load_string($res_data);
	$result = $xml->xpath($node);

	while(list( , $node) = each($result)) 
	{
		return $node;
	}
}

######################### alipay_notify ########################

class alipay_notify {
    var $gateway;           //网关地址
    var $_key;			  	//安全校验码
    var $partner;           //合作伙伴ID
    var $sign_type;         //签名方式 系统默认
    var $mysign;            //签名结果
    var $_input_charset;    //字符编码格式

    /**构造函数
	 * 从配置文件中初始化变量
	 * $partner 合作身份者ID
	 * $key 安全校验码
	 * $sign_type 签名类型
	 * $_input_charset 字符编码格式
     */
    function alipay_notify($partner,$key,$sign_type,$_input_charset) {
		$this->gateway = "http://wappaygw.alipay.com/service/rest.htm?";
		
        $this->partner          = $partner;
        $this->_key				= $key;
        $this->mysign           = "";
        $this->sign_type	    = $sign_type;
        $this->_input_charset   = $_input_charset;
    }

    /********************************************************************************/

    /**对notify_url的认证
	 *返回的验证结果：true/false
     */
    function notify_verify() {
		//判断POST来的数组是否为空
		if(empty($_POST)) {
			return false;
		}
		else {
			//此处为固定顺序，支付宝Notify返回消息通知比较特殊，这里不需要升序排列
			$notifyarray = array(
				"service"		=> $_POST['service'],
				"v"				=> $_POST['v'],
				"sec_id"		=> $_POST['sec_id'],
				"notify_data"	=> $_POST['notify_data']
			);

			$this->mysign = build_mysign($notifyarray,$this->_key,$this->sign_type);
			
			//记录日志（调试用）
			//log_result($this->mysign . ' ' . $_POST["sign"]);
	
			//判断veryfy_result是否为ture，生成的签名结果mysign与获得的签名结果sign是否一致
			//mysign与sign不等，与安全校验码、请求时的参数格式（如：带自定义参数等）、编码格式有关
			if ($this->mysign == $_POST["sign"]) 
			{
				return true;
			} 
			else 
			{
				return false;
			}
		}
    }

    /********************************************************************************/

    /**对return_url的认证
	 *return 验证结果：true/false
     */
    function return_verify() {
        //判断GET来的数组是否为空
		if(empty($_GET)) {
			return false;
		}
		else {
			$get          = para_filter($_GET);	    //对所有GET反馈回来的数据去空
			$sort_get     = arg_sort($get);		    //对所有GET反馈回来的数据排序
			$this->mysign = build_mysign($sort_get,$this->_key,$this->sign_type);    //生成签名结果
	
			if ($this->mysign == $_GET["sign"]) {            
				return true;
			}else {
				return false;
			}
		}
    }
}

######################### alipay_service ########################

class alipay_service {
	// 线上
	var $gateway_order = "http://wappaygw.alipay.com/service/rest.htm?";
	
	var $_key; // 安全校验码
	var $mysign; // 签名结果
	var $sign_type; // 签名类型 相当于config文件中的sec_id
	var $parameter; // 需要签名的参数数组
	var $format; // 字符编码格式
	var $req_data = ''; // post请求数据
	
	/**
	 * 构造函数
	 */
	function alipay_service() {
	}
	
	/**
	 * 创建alipay.wap.trade.create.direct接口
	 */
	function alipay_wap_trade_create_direct($parameter, $key, $sign_type) {
		$this->_key = $key; // MD5校验码
		$this->sign_type = $sign_type; // 签名类型，此处为MD5
		$this->parameter = para_filter ( $parameter ); // 除去数组中的空值和签名参数
		$this->req_data = $parameter ['req_data'];
		$this->format = $this->parameter ['format']; // 编码格式，此处为utf-8
		$sort_array = arg_sort ( $this->parameter ); // 得到从字母a到z排序后的签名参数数组
		$this->mysign = build_mysign ( $sort_array, $this->_key, $this->sign_type ); // 生成签名
		$this->req_data = create_linkstring ( $this->parameter ) . '&sign=' . urlencode ( $this->mysign ); // 配置post请求数据，注意sign签名需要urlencode
		                                                                                          
		// Post提交请求
		$result = $this->post ( $this->gateway_order );
		
		// 调用GetToken方法，并返回token
		return $this->getToken ( $result );
	}
	
	/**
	 * 调用alipay_Wap_Auth_AuthAndExecute接口
	 */
	function alipay_Wap_Auth_AuthAndExecute($parameter, $key) {
		$this->parameter = para_filter ( $parameter );
		$sort_array = arg_sort ( $this->parameter );
		$this->sign_type = $this->parameter ['sec_id'];
		$this->_key = $key;
		$this->mysign = build_mysign ( $sort_array, $this->_key, $this->sign_type );
		$RedirectUrl = $this->gateway_order . create_linkstring ( $this->parameter ) . '&sign=' . urlencode ( $this->mysign );
		
		// 跳转至该地址
		Header ( "Location: $RedirectUrl" );
	}
	
	/**
	 * 返回token参数
	 * 参数 result 需要先urldecode
	 */
	function getToken($result) {
		$result = urldecode ( $result ); // URL转码
		$Arr = explode ( '&', $result ); // 根据 & 符号拆分
		
		$temp = array (); // 临时存放拆分的数组
		$myArray = array (); // 待签名的数组
		                    // 循环构造key、value数组
		for($i = 0; $i < count ( $Arr ); $i ++) {
			$temp = explode ( '=', $Arr [$i], 2 );
			$myArray [$temp [0]] = $temp [1];
		}
		
		$sign = $myArray ['sign']; // 支付宝返回签名
		$myArray = para_filter ( $myArray ); // 拆分完毕后的数组
		
		$sort_array = arg_sort ( $myArray ); // 排序数组
		$this->mysign = build_mysign ( $sort_array, $this->_key, $this->sign_type ); // 构造本地参数签名，用于对比支付宝请求的签名
		
		if ($this->mysign == $sign) 		// 判断签名是否正确
		{
			return getDataForXML ( $myArray ['res_data'], '/direct_trade_create_res/request_token' ); // 返回token
		} else {
			echo ('签名不正确'); // 当判断出签名不正确，请不要验签通过
			return '签名不正确';
		}
	}
	
	/**
	 * PHP Crul库 模拟Post提交至支付宝网关
	 * 如果使用Crul 你需要改一改你的php.ini文件的设置，找到php_curl.dll去掉前面的";"就行了
	 * 返回 $data
	 */
	function post($gateway_url) {
		$ch = curl_init ();
		curl_setopt ( $ch, CURLOPT_URL, $gateway_url ); // 配置网关地址
		curl_setopt ( $ch, CURLOPT_HEADER, 0 ); // 过滤HTTP头
		curl_setopt ( $ch, CURLOPT_RETURNTRANSFER, 1 );
		curl_setopt ( $ch, CURLOPT_POST, 1 ); // 设置post提交
		curl_setopt ( $ch, CURLOPT_POSTFIELDS, $this->req_data ); // post传输数据
		$data = curl_exec ( $ch );
		curl_close ( $ch );
		return $data;
	}
}





?>